
#
#           |          _                     _______________________
#           |\       _/ \_                  |                       |
#           | \_    /_    \_                |    Alexandre Botao    |
#           \   \__/  \__   \               |     www.botao.org     |
#            \_    \__/  \_  \              |    55-11-8244-UNIX    |
#              \_   _/     \ |              |  alexandre@botao.org  |
#                \_/        \|              |_______________________|
#                            |
#

#   ______________________________________________________________________
#  |                                                                      |
#  |  This (Makefile) code is part of 'VDT' (the Visual Disk Test).       |
#  |                                                                      |
#  |  This code is free software: you can redistribute it and/or modify   |
#  |  it under the terms of the GNU General Public License as published   |
#  |  by the Free Software Foundation, either version 3 of the License,   |
#  |  or (at your option) any later version.                              |
#  |                                                                      |
#  |  This code is distributed in the hope that it will be useful,        |
#  |  but WITHOUT ANY WARRANTY; without even the implied warranty of      |
#  |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                |
#  |  See the GNU General Public License for more details.                |
#  |                                                                      |
#  |  You should have received a copy of the GNU General Public License   |
#  |  along with this code.  If not, see <http://www.gnu.org/licenses/>,  |
#  |  or write to the Free Software Foundation, Inc.,                     |
#  |  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.            |
#  |______________________________________________________________________|
#

SRC	=	vdt.c stdmisc.c stdtime.c stdstr.c stdmem.c
SIM	=	simio.c

VDTFLG	=	-DSYSIO -o vdt $(SRC) -lm -s
GNUFLG	=	-Wall -Wextra -O3 -D_GNU_SOURCE
LFSFLG	=	`getconf LFS_CFLAGS`

LNXFLG	=	$(GNUFLG) -DANSI $(LFSFLG) $(VDTFLG)

vdt: curses.ld
	@$(MAKE) compile_`uname`

curses.ld:
	./configure

compile_AIX:
	$(CC) -O -D_LARGE_FILES -DSYSIO -o vdt $(SRC) -lcurses -lm -Wl,-bmaxdata:0x80000000 -Wl,-blibpath:/opt/freeware/lib:/usr/lib:/lib

compile_AIX_alt:
	$(CC) -D_LARGE_FILES -DSYSIO -o vdt $(SRC) -lcurses -lm -Wl,-bmaxdata:0x80000000

compile_CYGWIN_NT-6.1:
	$(CC) -D_GNU_SOURCE -DSYSIO -Wall -Wextra -O3 -o vdt $(SRC) -lncurses -lm -s

compile_FreeBSD:
	$(CC) -D_GNU_SOURCE -DSYSIO -Wall -Wextra -O3 -o vdt $(SRC) -lncurses -lm -s

compile_NetBSD:
	$(CC) -D_GNU_SOURCE -DSYSIO -Wall -O3 -o vdt $(SRC) -lcurses -lm -s

compile_OpenBSD:
	$(CC) -D_GNU_SOURCE -DSYSIO -Wall -O3 -o vdt $(SRC) -lncurses -lm -s

compile_HP-UX:
	$(CC) +O3 -I/usr/local/include -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE -DSYSIO -o vdt $(SRC) -L/usr/local/lib -L/usr/local/lib/hpux32 `cat curses.ld` -lm -s

compile_HP-UX_alt:
	$(CC) -D_FILE_OFFSET_BITS=64 -D_LARGEFILE64_SOURCE -DSYSIO -o vdt $(SRC) `cat curses.ld` -lm -s

compile_Linux:
	$(CC) $(LNXFLG) -lncurses

compile_SunOS:
	$(CC) `getconf LFS_CFLAGS` -DSYSIO -o vdt $(SRC) `cat curses.ld` -lm -s

test check:vdt
	./vdtest.sh 32M

bigtest bigcheck: vdt
	./bigtest.sh 32M

deadtest deadcheck: vdt
	./deadtest.sh 32M

install: vdt
	chmod 755 vdt
	chmod 644 vdt.1
	[ -d /usr/local/bin ] || mkdir -p -m 755 /usr/local/bin
	[ -d /usr/local/man/man1 ] || mkdir -p -m 755 /usr/local/man/man1
	$(MAKE) install_`uname`

install_AIX:
	cp vdt /usr/local/bin
	ls -l /usr/local/bin/vdt
	cp vdt.1 /usr/share/man/man1
	ls -l /usr/share/man/man1/vdt.1

install_HP-UX:
	cp vdt /usr/local/bin
	ls -l /usr/local/bin/vdt
	cp vdt.1 /usr/local/man/man1
	chmod a+r /usr/local/man/man1/vdt.1
	ls -l /usr/local/man/man1/vdt.1

install_Linux:
	cp vdt /usr/local/bin
	ls -l /usr/local/bin/vdt
	cp vdt.1 /usr/share/man/man1
	ls -l /usr/share/man/man1/vdt.1

install_SunOS:
	cp vdt /usr/local/bin
	ls -l /usr/local/bin/vdt
	cp vdt.1 /usr/local/share/man/man1
	ls -l /usr/local/share/man/man1/vdt.1

clean:
	rm -f *.o *.lix vdt curses.ld vdt*.log vdt*.err vdt*.exe
	rm -f fst.o fst

unconf:
	@$(MAKE) clean
	rm -f Makefile config.h

fst:	fst.c
	cc -Wall -Wextra fst.c -o fst

#	cc -D{<OSID>|<OSTYP>} -D<IOTYP> -o [sim]vdt $(SRC) [$(SIM)] -l[n]curses -lm -s [-static] [-lpanel -lmenu -lforms]

# vi:nu ts=8
